/*
 * SM_StringMetric.java
 *
 * Created on 20 November 2015
 *
 * Version 1.0.1
 *
 * Copyright (c) Kieran Greer
 */

package org.licas.ai.solver.wrapper;


import org.licas.ai.solver.util.SM_HeuristicConst;
import org.ai_heuristic.eval.metric.*;
import org.ai_heuristic.functs.Function;
import org.ai_heuristic.functs.FunctionMetric;
import org.ai_heuristic.util.AiHeuristicConst;
import org.ai_heuristic.util.AiTypeConst;
import org.simmetrics.StringMetric;
import org.simmetrics.metrics.StringMetrics;

import java.util.ArrayList;


/**
 * Wrapper and interface for using the Simmetrics similarity measure metrics.
 * This uses the Simmetrics {@code StringMetric} class to pass the data onto the
 * simmetrics evaluator.
 * @author Kieran Greer
 */
public class SM_StringMetric extends FunctionMetric
{
    /** The related simpack metric */
    private String metricType;
    
    
    /**
     * Create a new instance of SM_StringMetric.
     * @param thisMetricType the simmetrics metric type.
     * @throws java.lang.Exception any error.
     */
    public SM_StringMetric(String thisMetricType) throws Exception
    {
        super();
        metricType = thisMetricType;
        initialiseValues();
    }
    
    /**
     * Initialise the function values, setting the config parameters or other.
     * @throws java.lang.Exception any error.
     */
    protected void initialiseValues() throws Exception
    {
        valueType = AiTypeConst.STRING;
    }
    
    /**
     * Evaluate the comparison between the two data lists and return the result.
     * This version expects each list to have only one object of type String, and then 
     * use them with the simmetrics function.
     * @param ds1 first value dataset.
     * @param ds2 second value dataset.
     * @return the result of applying the operator to the values.
     * @throws java.lang.Exception any error.
     */
    public ReplySet evaluate(MetricDataset ds1, MetricDataset ds2) throws Exception
    {
        double result;                          //evaluation result
        ArrayList dataList1, dataList2;            //data lists
        Object val1;                            //first value
        Object val2;                            //second value
        
        result = -1;
        checkValueType(ds1);
        dataList1 = ds1.getDataList();
        dataList2 = ds2.getDataList();
        
        //check if string values or other
        if ((dataList1.size() == 1) && (dataList2.size() == 1))
        {
            if (ds1.getValueType().equalsIgnoreCase(AiTypeConst.STRING) &&
                ds2.getValueType().equalsIgnoreCase(AiTypeConst.STRING))
            {
                val1 = innerObject(dataList1.get(0));
                val2 = innerObject(dataList2.get(0));
                result = evaluate(((String)val1).trim(), ((String)val2).trim());
            }
        }
        
        
        return new ReplySet(new Double(result));
    }
    
    /**
     * Return a value based on the function evaluation. This returns the similarity 
     * evaluation between the two strings specifically, which is the traditional
     * use of the function.
     * @param compOne the first comparison string.
     * @param compTwo the second comparison string.
     * @return the similarity evaluation double value.
     * @throws java.lang.Exception any error.
     */
    public double evaluate(String compOne, String compTwo)  throws Exception
    {
        float result;                           //evaluation result
        StringMetric simMetric;                 //the sim metric
        
        simMetric = createMetric();
        result = simMetric.compare(compOne, compTwo);
        
        return (double)result;
    }
    
    /**
     * Crete a copy of the metric to use.
     * @return the created metric.
     */
    private StringMetric createMetric()
    {
        if (metricType.equals(SM_HeuristicConst.SM_BLOCKDISTANCE))
        {
            return StringMetrics.blockDistance();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_COSINESIMILARITY))
        {
            return StringMetrics.cosineSimilarity();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_DEMERAULEVENSHTEIN))
        {
            return StringMetrics.damerauLevenshtein();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_JARO))
        {
            return StringMetrics.jaro();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_JAROWINKLER))
        {
            return StringMetrics.jaroWinkler();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_LEVENSHTEIN))
        {
            return StringMetrics.levenshtein();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_NEEDLEMANWUNCH))
        {
            return StringMetrics.needlemanWunch();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_SMITHWATERMAN))
        {
            return StringMetrics.smithWaterman();
        }
        else if (metricType.equals(SM_HeuristicConst.SM_SMITHWATERMANGOTOH))
        {
            return StringMetrics.smithWatermanGotoh();
        }
        
        return null;
    }

    /**
     * Return true if value2 is better than value1, as determined by the measurements of 
     * this evaluation function.
     * This version checks several simmetrics metric types before returning a value.
     * @param valueType the java type of the values to be evaluated.
     * @param value1 the first value type.
     * @param value2 the second value type.
     * @return true if value2 is better than value1, as determined by the simmetric function.
     * @throws java.lang.Exception any error.
     */
    public boolean isBetter(String valueType, Object value1, Object value2) throws Exception
    {
        if (metricType.equals(SM_HeuristicConst.SM_BLOCKDISTANCE) ||
            metricType.equals(SM_HeuristicConst.SM_COSINESIMILARITY) ||
            metricType.equals(SM_HeuristicConst.SM_DEMERAULEVENSHTEIN) ||
            metricType.equals(SM_HeuristicConst.SM_JARO) ||
            metricType.equals(SM_HeuristicConst.SM_JAROWINKLER) ||
            metricType.equals(SM_HeuristicConst.SM_LEVENSHTEIN) ||
            metricType.equals(SM_HeuristicConst.SM_NEEDLEMANWUNCH) ||
            metricType.equals(SM_HeuristicConst.SM_SMITHWATERMAN) ||
            metricType.equals(SM_HeuristicConst.SM_SMITHWATERMANGOTOH))
        {
            return eval.mathCompare(valueType, value2, value1, AiHeuristicConst.LT);
        }
        
        return false;
    }
    
    /**
     * Return true if a smaller distance between the two vectors is better.
     * This version checks several simmetrics metric types before returning a value.
     * @return true if a smaller evaluation is better.
     */
    public boolean sib()
    {
        if (metricType.equals(SM_HeuristicConst.SM_BLOCKDISTANCE) ||
            metricType.equals(SM_HeuristicConst.SM_COSINESIMILARITY) ||
            metricType.equals(SM_HeuristicConst.SM_DEMERAULEVENSHTEIN) ||
            metricType.equals(SM_HeuristicConst.SM_JARO) ||
            metricType.equals(SM_HeuristicConst.SM_JAROWINKLER) ||
            metricType.equals(SM_HeuristicConst.SM_LEVENSHTEIN) ||
            metricType.equals(SM_HeuristicConst.SM_NEEDLEMANWUNCH) ||
            metricType.equals(SM_HeuristicConst.SM_SMITHWATERMAN) ||
            metricType.equals(SM_HeuristicConst.SM_SMITHWATERMANGOTOH))
        {
            return true;
        }
        
        return true;
    }

    /**
     * Create and return a new instance of the function, initialised with this function's
     * value type {@code valueType} and math evaluator {@code mathCompare}.
     * @return a new instance of this function.
     * @throws java.lang.Exception any error.
     */
    public FunctionMetric newInstance() throws Exception
    {
        Function newInstance;                       //new instance
        
        newInstance = new SM_StringMetric(metricType);
        newInstance.setEvaluator(eval);
        
        return (FunctionMetric)newInstance;
    }
}
